﻿using C1.C1Excel;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Excel_RowHeightC
{
    public partial class Form1 : Form
    {
        C1.C1Excel.C1XLBook book;
        public Form1()
        {
            InitializeComponent();
        }

        public void SaveNewExcel(C1.C1Excel.C1XLBook book)
        {
            //Create a merged cell
            XLCellRange _RowRange = new C1.C1Excel.XLCellRange(0, 1, 1, 2);
            book.Sheets[0].MergedCells.Add(_RowRange);

            C1.C1Excel.XLSheet sheet = book.Sheets[0];
            //Wrap the text in a merged cell
            XLStyle style = new XLStyle(book);
            style.WordWrap = true;
            sheet[0, 1].Style = style;

            //Code to adjust the row height
            int rowHeight = sheet.DefaultRowHeight;
            int colWidth = sheet.DefaultColumnWidth;

            using (Graphics g = Graphics.FromHwnd(IntPtr.Zero))
            {
                foreach (XLCellRange cr in sheet.MergedCells)
                {
                    // get value (unformatted at this point)
                    var value = cr.Value;
                    if (value == null || value.ToString().Length == 0) continue; // sanity value
                    var text = value.ToString();

                    // calculation merged cell size
                    int width = 0, height = 0, lastHeight = sheet.DefaultRowHeight;

                    for (int i = cr.ColumnFrom; i <= cr.ColumnTo; i++)
                    {
                        width += C1XLBook.TwipsToPixels(sheet.Columns[i].Width > 0 ? sheet.Columns[i].Width : sheet.DefaultColumnWidth);
                    }

                    for (int i = cr.RowFrom; i <= cr.RowTo; i++)
                    {
                        lastHeight = sheet.Rows[i].Height > 0 ? sheet.Rows[i].Height : sheet.DefaultRowHeight;
                        height += C1XLBook.TwipsToPixels(lastHeight);
                    }

                    // format value if cell has a style with format set
                    var s = sheet.MergedCells[0].Style;
                    if (s != null && s.Format.Length > 0 && value is IFormattable)
                    {
                        string fmt = XLStyle.FormatXLToDotNet(s.Format);
                        text = ((IFormattable)value).ToString(fmt, CultureInfo.CurrentCulture);
                    }

                    // get font (default or style)
                    var font = (s != null && s.Font != null) ? s.Font : book.DefaultFont;

                    // measuring text
                    var sz = Size.Ceiling(g.MeasureString(text + (s.WordWrap ? Environment.NewLine : string.Empty) + "XX", font, width));

                    // correct last height
                    var delta = C1XLBook.PixelsToTwips(sz.Height - height);
                    if (lastHeight + delta >= 0)
                    {
                        sheet.Rows[cr.RowTo].Height = lastHeight + delta;
                    }
                }
            }


            //Save the modified Excel file
            book.Save("../../C1Sample.xls");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SaveNewExcel(book);
            MessageBox.Show("Excel file saved");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            book = new C1.C1Excel.C1XLBook();
            book.Load("../../Sample.xls");
            MessageBox.Show("Excel file loaded");
        }
    }
}
